/*
 * eID Chrome project.
 *
 * Copyright 2017-2021 e-Contract.be BV. All rights reserved.
 * e-Contract.be BV proprietary/confidential. Use is subject to license terms.
 */

"use strict";

function _base64ToUint8Array(base64) {
    var binary_string = window.atob(base64);
    var len = binary_string.length;
    var bytes = new Uint8Array(len);
    for (var i = 0; i < len; i++) {
        bytes[i] = binary_string.charCodeAt(i);
    }
    return bytes;
}

function loadCertificate(encodedCertificate) {
    var certificateData = _base64ToUint8Array(encodedCertificate);
    var certBuffer = forge.util.binary.raw.encode(certificateData);
    var certAsn1 = forge.asn1.fromDer(certBuffer, false);
    var cert = forge.pki.certificateFromAsn1(certAsn1);
    return cert;
}

function trimRight(data) {
    var idx;
    for (idx = 0; idx < data.length; idx++) {
        if (0 === data.charCodeAt(idx)) {
            break;
        }
    }
    var trimmedData = data.substring(0, idx);
    return trimmedData;
}

function contentScriptListener(message, sender, sendResponse) {
    // for the moment we simply parse the data files
    if (message.type === "validate-identity") {
        let encodedIdentityFile = message.encodedIdentityFile;
        let identityFile = _base64ToUint8Array(encodedIdentityFile);
        let identity = parseIdentityFile(identityFile);
        let responseMessage = {
            status: "OK",
            identity: identity
        };
        if (typeof message.encodedAddressFile !== "undefined") {
            let encodedAddressFile = message.encodedAddressFile;
            let addressFile = _base64ToUint8Array(encodedAddressFile);
            let address = parseAddressFile(addressFile);
            responseMessage.address = address;
        }
        sendResponse(responseMessage);
        // we have to return true here to indicate async processing
        return true;
    }
    // next is no longer activated for the moment
    if (message.type === "validate-identity") {
        var TRUSTED_PUBLIC_KEYS = new Map();
        async.each(ROOTS_CONFIG, function (rootConfig, callback) {
            var url = chrome.runtime.getURL("roots/" + rootConfig);
            var xhr = new XMLHttpRequest();
            xhr.open("GET", url, true);
            xhr.responseType = "arraybuffer";
            xhr.onload = function () {
                if (this.status === 200) {
                    var certData = forge.util.binary.raw.encode(new Uint8Array(this.response));
                    var certAsn1 = forge.asn1.fromDer(certData, false);
                    var cert = forge.pki.certificateFromAsn1(certAsn1);
                    var subjectKeyIdentifier = getSubjectKeyIdentifier(cert);
                    TRUSTED_PUBLIC_KEYS.set(subjectKeyIdentifier, cert.publicKey);
                    callback();
                }
            };
            xhr.send();
        }, function () {
            var encodedCertificates = message.encodedCertificates;
            var certificates = [];
            for (var idx = 0; idx < encodedCertificates.length; idx++) {
                var encodedCertificate = encodedCertificates[idx];
                var certificate = loadCertificate(encodedCertificate);
                certificates[idx] = certificate;
            }

            var encodedIdentitySignatureFile = message.encodedIdentitySignatureFile;
            var identitySignatureFile = forge.util.decode64(encodedIdentitySignatureFile);

            var encodedIdentityFile = message.encodedIdentityFile;
            var identityFile = _base64ToUint8Array(encodedIdentityFile);

            var rootCertificate = certificates[1];

            var rootSubjectKeyIdentifier = getSubjectKeyIdentifier(rootCertificate);
            if (!TRUSTED_PUBLIC_KEYS.has(rootSubjectKeyIdentifier)) {
                console.error("untrusted root public key identifier: " + rootSubjectKeyIdentifier);
                console.log("trusted public keys: %o", TRUSTED_PUBLIC_KEYS);
                var responseMessage = {
                    status: "UNTRUSTED_ROOT_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }
            var trustedPublicKey = TRUSTED_PUBLIC_KEYS.get(rootSubjectKeyIdentifier);
            var trustedPublicKeyPem = forge.pki.publicKeyToPem(trustedPublicKey);
            var rootCertificatePublicKeyPem = forge.pki.publicKeyToPem(rootCertificate.publicKey);
            if (trustedPublicKeyPem !== rootCertificatePublicKeyPem) {
                console.error("public key mismatch");
                var responseMessage = {
                    status: "UNTRUSTED_ROOT_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }

            var rootSelfSigned = rootCertificate.verify(rootCertificate);
            if (!rootSelfSigned) {
                var responseMessage = {
                    status: "ROOT_SELF_SIGNED_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }

            var rrnCertificate = certificates[0];

            var rootSigned = rootCertificate.verify(rrnCertificate);
            if (!rootSigned) {
                var responseMessage = {
                    status: "ROOT_SIGNATURE_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }

            var md;
            if (rrnCertificate.signatureOid === "1.2.840.113549.1.1.5") {
                md = forge.md.sha1.create();
            } else if (rrnCertificate.signatureOid === "1.2.840.113549.1.1.11") {
                md = forge.md.sha256.create();
            }
            md.start();
            md.update(forge.util.decode64(encodedIdentityFile));
            var rrnVerifyResult = rrnCertificate.publicKey.verify(md.digest().getBytes(), identitySignatureFile);
            if (!rrnVerifyResult) {
                var responseMessage = {
                    status: "RRN_IDENTITY_SIGNATURE_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }

            var identity = parseIdentityFile(identityFile);

            var now = new Date();
            if (now > identity.cardValidityDateEnd) {
                var responseMessage = {
                    status: "EID_EXPIRED_ERROR"
                };
                sendResponse(responseMessage);
                return;
            }

            identity.dateOfBirthTime = identity.dateOfBirth.getTime();

            if (typeof message.encodedAddressFile !== "undefined") {
                var encodedAddressFile = message.encodedAddressFile;

                var encodedAddressSignatureFile = message.encodedAddressSignatureFile;
                var addressSignatureFile = forge.util.decode64(encodedAddressSignatureFile);

                var md;
                if (rrnCertificate.signatureOid === "1.2.840.113549.1.1.5") {
                    md = forge.md.sha1.create();
                } else if (rrnCertificate.signatureOid === "1.2.840.113549.1.1.11") {
                    md = forge.md.sha256.create();
                }
                md.start();

                var sigAddressFile = forge.util.decode64(encodedAddressFile);
                var trimmedSigAddressFile = trimRight(sigAddressFile);
                md.update(trimmedSigAddressFile);

                md.update(identitySignatureFile);
                var rrnVerifyAddressResult = rrnCertificate.publicKey.verify(md.digest().getBytes(), addressSignatureFile);
                if (!rrnVerifyAddressResult) {
                    var responseMessage = {
                        status: "RRN_ADDRESS_SIGNATURE_ERROR"
                    };
                    sendResponse(responseMessage);
                    return;
                }

                var addressFile = _base64ToUint8Array(encodedAddressFile);
                var address = parseAddressFile(addressFile);
            }

            var responseMessage = {
                status: "OK",
                identity: identity
            };
            if (typeof message.encodedAddressFile !== "undefined") {
                responseMessage.address = address;
            }
            sendResponse(responseMessage);
        });
        // we have to return true here to indicate async processing
        return true;
    }
}

function getSubjectKeyIdentifier(certificate) {
    for (var idx = 0; idx < certificate.extensions.length; idx++) {
        var extension = certificate.extensions[idx];
        if (extension.id === "2.5.29.14") {
            return extension.subjectKeyIdentifier;
        }
    }
}

function init() {
    chrome.runtime.onMessage.addListener(contentScriptListener);
}

init();
