/*
 * eID Chrome project.
 *
 * Copyright 2015-2017 e-Contract.be BVBA. All rights reserved.
 * e-Contract.be BVBA proprietary/confidential. Use is subject to license terms.
 */

"use strict";

function convertToString(value) {
    return String.fromCharCode.apply(null, value);
}

function toKey(value) {
    var key = value[0] & 0x0F;
    if (2 === value.length) {
        key *= 10;
        key += value[1] & 0x0F;
    }
    return key;
}

function convertToDocumentType(value) {
    var key = toKey(value);
    switch (key) {
        case 1:
            return "BELGIAN_CITIZEN";
        case 6:
            return "KIDS_CARD";
        case 7:
            return "BOOTSTRAP_CARD";
        case 8:
            return "HABILITATION_CARD";
        case 11:
            return "FOREIGNER_A";
        case 12:
            return "FOREIGNER_B";
        case 13:
            return "FOREIGNER_C";
        case 14:
            return "FOREIGNER_D";
        case 15:
            return "FOREIGNER_E";
        case 16:
            return "FOREIGNER_E_PLUS";
        case 17:
            return "FOREIGNER_F";
        case 18:
            return "FOREIGNER_F_PLUS";
        case 19:
            return "EUROPEAN_BLUE_CARD_H";
    }
}

function convertValidityDate(value) {
    var stringValue = new TextDecoder("utf-8").decode(value);
    var day = parseInt(stringValue.substring(0, 2));
    var month = parseInt(stringValue.substring(3, 5));
    var year = parseInt(stringValue.substring(6));
    var date = new Date(year, month - 1, day);
    return date;
}

var MONTHS = [
    ["JAN"],
    ["FEV", "FEB"],
    ["MARS", "MAAR", "MÄR"],
    ["AVR", "APR"],
    ["MAI", "MEI"],
    ["JUIN", "JUN"],
    ["JUIL", "JUL"],
    ["AOUT", "AUG"],
    ["SEPT", "SEP"],
    ["OCT", "OKT"],
    ["NOV"],
    ["DEC", "DEZ"]
];

function toMonth(monthStr) {
    monthStr = monthStr.trim();
    for (var monthIdx = 0; monthIdx < MONTHS.length; monthIdx++) {
        var monthNames = MONTHS[monthIdx];
        for (var monthNameIdx = 0; monthNameIdx < monthNames.length; monthNameIdx++) {
            var monthName = monthNames[monthNameIdx];
            if (monthName === monthStr) {
                return monthIdx;
            }
        }
    }
}

function convertDateOfBirth(value) {
    var dateOfBirthStr = new TextDecoder("utf-8").decode(value);
    var spaceIdx = dateOfBirthStr.indexOf('.');
    if (-1 === spaceIdx) {
        spaceIdx = dateOfBirthStr.indexOf(' ');
    }
    if (spaceIdx > 0) {
        var dayStr = dateOfBirthStr.substring(0, spaceIdx);
        var day = parseInt(dayStr);
        var monthStr = dateOfBirthStr.substring(spaceIdx + 1, dateOfBirthStr.length - 4 - 1);
        if (monthStr.endsWith(".")) {
            monthStr = monthStr.substring(0, monthStr.length - 1);
        }
        var month = toMonth(monthStr);
        var yearStr = dateOfBirthStr.substring(dateOfBirthStr.length - 4);
        var year = parseInt(yearStr);
        return new Date(year, month, day);
    }
    if (dateOfBirthStr.length() === 4) {
        return new Date(parseInt(dateOfBirthStr), 0, 1);
    }
}

function convertGender(value) {
    var genderStr = new TextDecoder("utf-8").decode(value);
    if (genderStr === "M") {
        return "MALE";
    }
    if (genderStr === "F") {
        return "FEMALE";
    }
    if (genderStr === "V") {
        return "FEMALE";
    }
    if (genderStr === "W") {
        return "FEMALE";
    }
}

var BEID_IDENTITY_METADATA = [
    {
        tag: 1,
        fieldName: "cardNumber",
        convertor: convertToString
    },
    {
        tag: 3,
        fieldName: "cardValidityDateBegin",
        convertor: convertValidityDate
    },
    {
        tag: 4,
        fieldName: "cardValidityDateEnd",
        convertor: convertValidityDate
    },
    {
        tag: 6,
        fieldName: "nationalNumber",
        convertor: convertToString
    },
    {
        tag: 7,
        fieldName: "name",
        convertor: convertToString
    },
    {
        tag: 8,
        fieldName: "firstName",
        convertor: convertToString
    },
    {
        tag: 10,
        fieldName: "nationality",
        convertor: convertToString
    },
    {
        tag: 11,
        fieldName: "placeOfBirth",
        convertor: convertToString
    },
    {
        tag: 12,
        fieldName: "dateOfBirth",
        convertor: convertDateOfBirth
    },
    {
        tag: 13,
        fieldName: "gender",
        convertor: convertGender
    },
    {
        tag: 15,
        fieldName: "documentType",
        convertor: convertToDocumentType
    }
];

var BEID_ADDRESS_METADATA = [
    {
        tag: 1,
        fieldName: "streetAndNumber",
        convertor: convertToString
    },
    {
        tag: 2,
        fieldName: "zip",
        convertor: convertToString
    },
    {
        tag: 3,
        fieldName: "municipality",
        convertor: convertToString
    }
];

function parseBeIDFile(file, metadata) {
    var indexedMetadata = new Map();
    metadata.forEach(function (metadataEntry) {
        indexedMetadata.set(metadataEntry.tag, metadataEntry);
    });

    var result = {};
    var idx = 0;
    while (idx < file.byteLength - 1) {
        var tag = file[idx];
        idx++;
        var lengthByte = file[idx];
        var length = lengthByte & 0x7f;
        while ((lengthByte & 0x80) === 0x80) {
            idx++;
            lengthByte = file[idx];
            length = (length << 7) + (lengthByte & 0x7f);
        }
        idx++;
        if (0 === tag) {
            idx += length;
            continue;
        }
        if (indexedMetadata.has(tag)) {
            var metadataEntry = indexedMetadata.get(tag);
            var value = file.subarray(idx, idx + length);
            result[metadataEntry.fieldName] = metadataEntry.convertor(value);
        }
        idx += length;
    }
    return result;
}

function parseIdentityFile(file) {
    return parseBeIDFile(file, BEID_IDENTITY_METADATA);
}

function parseAddressFile(file) {
    return parseBeIDFile(file, BEID_ADDRESS_METADATA);
}